@@ -2,7 +2,7 @@
 #
 # Copyright 2001-2025 Sam Varshavchik.
 #
-# with contributions from David Cantrell.
+# with contributions from David Cantrell and brian d foy
 #
 # This program is free software; you can redistribute it
 # and/or modify it under the same terms as Perl itself.
@@ -50,7 +50,7 @@ use Carp;
 
 );
 
-$VERSION = "0.23";
+$VERSION = "0.26";
 
 1;
 
@@ -536,7 +536,7 @@ sub addr2cidr {
 	    $n &= 0xF8;
 	}
 
-	my $s=join(".", @ipcpy);
+	my $s=join(".", map { s/\A0+([0-9])/$1/; $_ } @ipcpy);
 
 	push @blocks, ($isipv6 ? _ipv4to6($s):$s) . "/$bits";
     }
@@ -1203,7 +1203,7 @@ sub cidrlookup {
 =head2 $ip=Net::CIDR::cidrvalidate($ip);
 
 Validate whether $ip is a valid IPv4 or IPv6 address, or a CIDR.
-Returns its argument or undef.
+Returns its validated argument or undef.
 Spaces are removed, and IPv6 hexadecimal address are converted to lowercase.
 
 $ip with less than four octets gets filled out with additional octets, and
@@ -1320,10 +1320,32 @@ sub cidrvalidate {
 
     $v =~ s/([0-9A-Fa-f]+)/_triml0($1)/ge;
 
-	my $compressed = _compress_ipv6($v);
-    foreach (addr2cidr($v))
+    my @compressed = _compress_ipv6($v);
+
+    unless ($compressed[0] =~ /::/)
     {
-	return $_ if $_ eq "$compressed/$suffix";
+	my @split = split(/:/, $compressed[0]);
+
+	foreach my $i (0..$#split)
+	{
+	    if ($split[$i] eq "0")
+	    {
+		my @cpy = @split;
+
+		$cpy[$i] = "";
+
+		push @compressed, join(":", @cpy);
+		last;
+	    }
+	}
+    }
+
+    foreach my $candidate (addr2cidr($v))
+    {
+	foreach my $c (@compressed)
+	{
+	    return $candidate if $candidate eq "$c/$suffix";
+	}
     }
     return undef;
 }
@@ -1352,6 +1374,7 @@ but the functions will croak if you're totally off the wall.
 Sam Varshavchik <sam@email-scan.com>
 
 With some contributions from David Cantrell <david@cantrell.org.uk>
+and brian d foy <briandfoy@pobox.com>.
 
 =cut
 
